/* Program cpfpred/predpkg converted to c:
 * 12/06/05 - rlr.
 * History:
 *  08/09/17 - Create proper prototypes, and expand severa arrays to prevent
 *             size mismatches that confuse certain compilers.
 *             Thanks to Nicolas MAURICE for finding this problem. rlr.
 *             Also, initialize arot_type tp 0.
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "intrng.h"
#include "obsinf.h"
#include "bounce.h"
#include "dirctn.h"
#include "../include/ephi.h"
#include "pnt.h"

double twopi, cdr, casr, ctsr;
int debug;
extern struct eh ephihdr;
extern struct ei ephi;
extern double feature[3];
extern double bodftr[1][6];
//void ephtel (double, double, double, double bodtel[6],
	//double bodftr[1][6], double trfvec[6], double rfrvec[6],
	//double *, double *, double *, double *,
void ephtel (double, double, double, double [6],
	double [1][6], double [6], double [6],
	double *, double *, double *, double *,
	double [6], double [6]);
void sphere (double [6], double [6]);
void rectng (double [6], double [6]);
void precess (double, double, double [3][6]);
void point (double, double [6], double [3], double [3],
       double, double, double [3],
       double [6], double [6], double *);
void matsub (double [6], double [6], double [6], int);
void matmul (double [3][6], double [3][6], double [3][6], int, int, int);
void matadd (double [6], double [6], double [6], int);
void littim (double, double, double, double,
	double, double, double *);
void librate (double [3], double [3][6]);
void cnvcoor (double, double [3], double [6]);
void clkcor (double, double [3], double *);
void calcrng (double, double, double [1][6], int,
	 double, double *, double *, double *,
	 double *, double *, double [3],
	 double [3], double [3]);
void cinterp_eop ();
void cinterp_off ();
void cinterp_pvac ();
void cinterp_rot ();
int rotmat (int, double *, double *);

/*-----------------------------------------------------------------------
C  Calculate round-trip range and point angles between earth site and 
C  ranging target.
C
C  Input
C	tjdint	- d.p. whole part of Julian date
C	secutc  - d.p. seconds of fractional Julian day
C	vlight	- d.p. velocity of light in km/sec
C	backang - logical flag: Compute return leg angles?
C  Output
C	ttltim	- d.p. corrected round-trip travel time in nsec
C	lhatrn  - telescope xmit local hour angle and declination 
C		  (d.p. m and degrees)
C	azaltrn - telescope xmit az and el (d.p. m and degrees)
C	ratrn	- telescope xmit right ascension (d.p. hours)
C	lharcv  - telescope xcv local hour angle and declination 
C		  (d.p. m and degrees)
C	azalrcv - telescope xcv az and el (d.p. m and degrees)
C	rarcv	- telescope xcv right ascension (d.p. hours)
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
predict (double tjdint, double sutct, double vlight, int backang,
	 double *ttltim, double lhatrn[6], double azaltrn[6],
	 double *ratrn, double lharcv[6], double azalrcv[6], double *rarcv)
{
  double aberout[3], aberback[3], rotang[3];
  double nrfrvec[6], naberback[3];
  double dtadt, dist12, dist23, corr12, corr23;
  int i;
/*-----------------------------------------------------------------*/
  /*debug = 1;*/
/*  Compute out-bound and in-bound distances and corrections */
  calcrng (tjdint, sutct, bodftr, backang, vlight,
	   &dtadt, &dist12, &dist23, &corr12, &corr23, aberout, aberback,
	   rotang);

/*  Compute two way light travel time and apply necessary corrections */
  if (debug)
    printf
      ("to littim d12 %18.10f d23 %18.10f c12 %10.2f c23 %10.2f dtadt %12.10f\n",
       dist12, dist23, corr12, corr23, dtadt - 1.0e0);
  littim (dist12, dist23, vlight, corr12, corr23, dtadt, ttltim);
  if (debug)
    printf ("from littim %18.10f %f\n", *ttltim, vlight);

/*  Compute pointing angles (transmitter) */
  legdir = 1;

  point (tjdint + sutct / 86400.e0,
	 trfvec, aberout, transm, tgdlat, tlong,
	 rotang, lhatrn, azaltrn, ratrn);

  if (debug)
    printf ("from t point %f %f %f  %f %f %f\n",
	    lhatrn[0], lhatrn[1], lhatrn[2], azaltrn[0], azaltrn[1],
	    azaltrn[2]);

  if (debug)
    printf ("from r point %f %f %f\n", lharcv[0], lharcv[1], lharcv[2]);

/*  If transmitter not receiver... */
  if (backang)
    {

/*  Compute pointing angles (receiver) */
      legdir = 2;

/*  Transponder test??? */
      for (i = 0; i < 6; i++)
	{
	  nrfrvec[i] = -rfrvec[i];
	  if (i < 3)
	    naberback[i] = -aberback[i];
	}

      point (tjdint + (sutct + *ttltim) / 86400.e0,
	     nrfrvec, naberback, receiv, rgdlat, rlong,
	     rotang, lharcv, azalrcv, rarcv);

      if (debug)
	printf ("from r point %f %f %f  %f %f %f\n",
		lharcv[0], lharcv[1], lharcv[2], azalrcv[0], azalrcv[1],
		azalrcv[2]);

    }

}

/*-----------------------------------------------------------------------
C  CALCRNG
C  Compute outleg and inleg distances and corrections
C
C  Input
C	tjdint	- d.p. whole part of Julian date
C	secutc  - d.p. seconds of fractional Julian day
C	backang - logical flag: Compute return leg angles?
C	vlight	- d.p. velocity of light in km/sec
C  Output
C	dtadt	- Scale factor to convert from atomic to coordinate time
C	dist12	- out-bound telescope to target distance (d.p. sec)
C	dist23	- in-bound tar to telescope distance (d.p. sec)
C	corr12	- out-bound leg relativistic correction (d.p. nsec)
C	corr23	- in-bound leg relativistic correction (d.p. nsec)
C	aberout - out-bound aberration correction (d.p. x, y, z m)
C	aberback- in-bound aberration correction (d.p. x, y, z m)
C	rotang	- rotation angle (euler or ...) (d.p. degrees)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
calcrng (double tjdint, double secutc, double bodftr[1][6], int backang,
	 double vlight, double *dtadt, double *dist12, double *dist23,
	 double *corr12, double *corr23, double aberout[3],
	 double aberback[3], double rotang[3])
{
  double tjdutc;
  double x, y, dutc;
  int i, ierr;

/*  Get the scale conversion between atomic and coordinate time */
  tjdutc = tjdint + secutc / 86400.e0;
  clkcor (tjdutc, transm, dtadt);
  if (debug)
    printf ("from clkcor, dtadt= %16.13f\n", *dtadt);

/*  Convert telescope coordinates from sperical to rectangular */
/*  MOVE TO INITIALIZATION... */
  cnvcoor (1.e0, transm, bodtrn);
  for (i = 0; i < 6; i++)
    {
      bodrcv[i] = bodtrn[i];
    }
/*     If transmit and receive telescopes are not the same... */
  if (backang)
    cnvcoor (1.e0, receiv, bodrcv);

/*  Get the current earth orientation parameters, if present */
/*  NOTE: THIS IS NOT CURRENTLY USED!! */
  if (ephihdr.ref_frame > 0 && ephi.nveop > 0)
    cinterp_eop (tjdint, secutc, &x, &y, &dutc, &ierr);

/*  Compute the outbound and inbound leg distances and corrections */
  if (debug)
    printf ("station: %f %f %f  %f %f %f", transm[0], transm[1], transm[2],
	    bodtrn[0], bodtrn[1], bodtrn[2]);
  ephtel (vlight, tjdint, secutc, bodtrn, bodftr,
	  trfvec, rfrvec, dist12, dist23, corr12, corr23, aberout, aberback);
  if (debug)
    printf
      ("EPHTEL OP: %f %f %f  %f %f %f  %f %f  %f %f  %f %f %f  %f %f %f\n",
       trfvec[0], trfvec[1], trfvec[2], rfrvec[0], rfrvec[1], rfrvec[2],
       *dist12, *dist23, *corr12, *corr23, aberout[0], aberout[1], aberout[2],
       aberback[0], aberback[1], aberback[2]);

/*  Convert scalar and vector distances from m to km */
  (*dist12) /= 1.e3;
  (*dist23) /= 1.e3;
  for (i = 0; i < 6; i++)
    {
      trfvec[i] /= 1.e3;
      rfrvec[i] /= 1.e3;
    }

  if (debug)
    printf ("dist12&23 %f %f corr12&23 %f %f\n",
	    *dist12, *dist23, *corr12, *corr23);

}

/*-----------------------------------------------------------------------
C  Compute the relativistic differential relationship, in seconds, to
C  convert an interval of proper or atomic time to an interval of
C  coordinate time (see Mulholland, 1972 PASP, 84, P.357.
C
C  This effect is about 1.3 nsec for the moon  but hundreds of nsec for 
C  Mars.
C
C  Input
C	tjdutc - Julian date
C	transm - Spherical coordinates of telescope (need geocentric lat
C		 and long.
C
C  Output
C	dtadt - scale factor: a bit more than 1
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
clkcor (double tjdutc, double transm[3], double *dtadt)
{
  double bd, bd2, bd3, dd, ecs, ems;
  double cosu, elong, phi, u, ut;

/*......................................*/
  dd = tjdutc - 2415020.e0;
  bd = dd * 1.0e-04;
  bd2 = bd * bd;
  bd3 = bd * bd2;
  ems =
    (358.475845e0 + 0.985600267e0 * dd - 1.12e-05 * bd2 -
     0.7e-07 * bd3) * cdr;
  ems = fmod (ems, twopi);
  ecs = 0.01675104e0 - 1.1444e-05 * bd - 9.4e-09 * bd2;

/*......................................*/
/*.....OBTAIN ECCENTRIC ANOMALY USING THE ABOVE PARAMETERS (SEE BROUWER */
/*.....AND CLEMENCE, P. 76) (same for mit and jpl ephemerides) */
  u = ems + ecs * sin (ems);
  ut = fmod (tjdutc + 0.5e0, 1.0e0);
  ut = ut * twopi;
  elong = transm[1] * cdr;
  phi = transm[2] * cdr;
  cosu = cos (u);
  *dtadt = 1.0e0 - 3.302e-10 * cosu / (1.0e0 - ecs * cosu) -
    1.476e-10 * cos (phi) * cos (ut + elong);

  if (debug)
    printf ("dtadt= %15.13f\n", *dtadt);
}

/*-----------------------------------------------------------------------
      SUBROUTINE cnvcoor (scale, sphpos, recpos)
      IMPLICIT DOUBLE PRECISION (a-h,o-z)
C  Convert coordinates from spherical to rectangular, using input
C  scale factor.

C  Input
C	scale  - scale factor. Aukm will convert from km to au; 1 will
C		 leave km in km.
C	sphpos - body centered distance in km, east longitude and
C		 latitude both in degrees
C  Output
C	recpos - rectangular body fixed body centered coordinates
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
cnvcoor (double aukm, double sphpos[3], double recpos[6])
{
  int i;
  double tmppos[6];

  tmppos[0] = sphpos[0] / aukm;
  tmppos[1] = sphpos[1] * cdr;
  tmppos[2] = sphpos[2] * cdr;
  for (i = 3; i < 6; i++)
    tmppos[i] = 0.0;
  rectng (tmppos, recpos);
}

/*-----------------------------------------------------------------------
C
C    Calculates local apparent sidereal time in terms of ut1 (i.e.
C    last1 in mueller's notation) at a place whose longitude is given
C    as the second argument. See pp. 50-53 of Explanatory Supplement to
C    the Astronomical Almanac (1992 ed.).
C    Note: this appears to be a low precision conversion... rlr.
C
C  Inputs
C      dj      - Julian date
C      long    - Longitude of site in radians
C  output
C      dlsid   - Local Mean Sidereal Time in radians
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
double
dlsid (double dj, double longi)
{
  double tj, tj2, tj3, rdpts, gstc, gst1, gst2, gst3, ddst;
  double rdlsid;

/*       Calculate the number of Julian centuries between the input date
 *       and 1900.0. */
  tj = (dj - 2451545.0e0) / 36525.0e0;
  tj2 = tj * tj;
  tj3 = tj2 * tj;

  rdpts = cdr / 3600.e0 * 15.e0;
  gstc = ((6.0e0 * 60.0e0 + 41.0e0) * 60.0e0 + 50.54841e0) * ctsr;
  gst1 = 8640184.812866e0 * ctsr;
  gst2 = 0.093104e0 * ctsr;
  gst3 = -6.2e-06 * ctsr;
  ddst = fmod (dj + 0.5e0, 1.e0) * twopi +
    longi * cdr + gstc + gst1 * tj + gst2 * tj2 + gst3 * tj3;
  rdlsid = fmod (ddst, twopi);
  if (rdlsid < 0.0e0)
    (rdlsid) += twopi;
  return (rdlsid);
}

/*-----------------------------------------------------------------------
C 
C  compute the telescope to target vector
C
C  For the out-bound leg, iterate the light travel time
C  from the telescope to the target. 
C    - Interpolate pvac at fire time (jdint,secutc)
C    - compute x,y,z telescope to target distance (stvec) from
C	geocenter-to-target (outvec)- telescope-to-geocenter (bodtel).
C    - compute the scalar distances corresponding to outvec and stvec.
C    - The difference between these scalar distance is then the light-time
C	delay because of the geocenter-to-telescope vector. So, backup
C	the ephemeris interpolation time at the earth's center by this
C	amount, so that the firing time is referred to the time at the
C	telescope. (In other words, we are firing from the telescope, 
C	not the earth's center, and that requires a change to the 
C	interpolation time.)
C    - Go back and interpolate with the new time.
C    - Continue until the change in time is "small".
C  For the in-bound leg, iterate the light travel time
C  from the telescope to the target. 
C    - Interpolate pvac at fire time (jdint,secutc+outleg length)
C    - compute x,y,z telescope to target distance (stvec) from
C	geocenter-to-target (outvec)- telescope-to-geocenter (bodtel).
C    - We know the interpolation time is the bounce time, which is already 
C	known from the outleg computation, so we are done - no iteration is
C	needed.
C  Save the corrections and return.
C
C  Input
C	vlight	- d.p. velocity of light in km/sec
C	jdint	- d.p. whole part of Julian date
C	secutc  - d.p. seconds of fractional Julian day
C	bodtel  - d.p. 6 element array containing telescope x, y, z 
C		  coordinates and velocities (w.r.t. geocenter)
C	bodftr  - d.p. 6 element array containing lunar target x, y, z 
C  Output   - d.p. 6 body-fixed state vector, telescope to target
C	trfvec  - d.p. 6 body-fixed state vector, telescope to target (out)
C	rfrvec  - d.p. 6 body-fixed state vector, target to telescope (back)
C	dist12	- outbound one way light travel time in seconds
C	dist23	- inbound one way light travel time in seconds
C	corr12	- outbound one way relativistic correction in nsec
C	corr23	- inbound  way relativistic correction in nsec
C	aberout - d.p. 3 element array containing aberration corrections
C		  in meters. (out)
C	aberback- d.p. 3 element array containing aberration corrections
C		  in meters. (back)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
ephtel (double vlight, double jdint, double secutc, double bodtel[6],
	double bodftr[1][6], double trfvec[6], double rfrvec[6],
	double *dist12, double *dist23, double *corr12, double *corr23,
	double aberout[3], double aberback[3])
{
  double olddt, corrdum;
  double outvec[6], backvec[6];
  double aberdum[3];
  double offsetout[3], offsetback[3];
  double agast[2], rotang[3];
  double gastm[3][6];
  double libtmp[3][6], celftr[3][6];
  double precm[3][6], ephftr[3][6], ephftrr[3][6], jdepoch = 2451545.e0;
  double rnger, ddist12, olddist12;
  double seci, dt;
  int arot_type= 0;
  int ierr;

/*  Apply outbound leg light travel time as needed... */
  dt = 0.e0;

  if (debug)
    printf ("ro %f\n", vlight);

/*l10:*/
  do
    {
/*    Back up the interpolation time at earth's center so that fire time for
 *    the station is the same as the input time to this routine, */
      olddt = dt;
      seci = secutc - dt;
      if (debug)
	printf ("seci: %f olddt: %f\n", seci, olddt);

/* Outbound */
      cinterp_pvac (jdint, seci, outvec, backvec,
		    aberout, aberdum, corr12, &corrdum, 1, &ierr);
      if (ierr != 0)
	exit(-1);

      matsub (outvec, bodtel, trfvec, 6);
      rnger =
	sqrt (outvec[0] * outvec[0] + outvec[1] * outvec[1] +
	      outvec[2] * outvec[2]);
      *dist12 =
	sqrt (trfvec[0] * trfvec[0] + trfvec[1] * trfvec[1] +
	      trfvec[2] * trfvec[2]);
      dt = (rnger - *dist12) / vlight / 1.e3;

      if (debug)
	printf ("et1 %f  %f %f %f  %f %f %f  %f %f %f  %f  %f  %f\n",
		seci, outvec[0], outvec[1], outvec[2],
		backvec[0], backvec[1], backvec[2],
		trfvec[0], trfvec[1], trfvec[2], rnger, *dist12, dt);

    }
/* want a variable tolerance? Then FIXME */
  while (fabs (dt - olddt) >= 1.e-10);

/*  Get the current offsets, if present. USE THEM: FIXME */
  if (ephi.nvoff > 0)
    {
      cinterp_off (jdint, seci, offsetout, offsetback, &ierr);
      if (ierr != 0)
	exit(-1);
    }

/* Get the current rotation angles, if present
 *   NOTE: For the moon, this give angles adequate for pointing
 *   but NOT for ranging, due to ~100 error from lack of nutations. */
  if (ephi.nvrot > 0)
    {
      cinterp_rot (jdint, seci + *dist12 / vlight / 1.e3,
		   &arot_type, &agast[0], rotang, &ierr);
      if (ierr != 0)
	exit(-1);

      agast[0] *= 15.e0 * cdr;
      agast[1] = 0.e0;

      if (arot_type == 1)
	{
/*     Create the libration matrix from euler angles */
	  librate (rotang, libtmp);

/*     Apply libration to lunar target */
	  matmul (libtmp, bodftr, celftr, 6, 3, 1);
	  precess (jdepoch,
		   jdint + (seci + *dist12 / vlight / 1.e3) / 86400.e0,
		   precm);
	  matmul (precm, celftr, ephftr, 6, 3, 1);

/*     Rotate from inertial to (earth) body fixed */
	  rotmat (-3, agast, gastm[0]);
	  matmul (gastm, ephftr, ephftrr, 6, 3, 1);
	  matadd (trfvec, ephftrr[0], trfvec, 6);

       if (debug)
         printf("bodftr %f %f %f\ncelftr %f %f %f\nephftr %f %f %f\n\
ephftrr %f %f %f\ntrfvec %f %f %f agast %f\n",
                bodftr[0][0],bodftr[0][1],bodftr[0][2],celftr[0][0],celftr[0][1],celftr[0][2],
                ephftr[0][0],ephftr[0][1],ephftr[0][2],ephftrr[0][0],ephftrr[0][1],ephftrr[0][2],
                trfvec[0],trfvec[1],trfvec[2],agast[0]);

	}
/* recompute range */
      if (debug)
	printf ("old dist12 %f, old dt: %f\n", *dist12, olddt);
      olddist12 = *dist12;
      *dist12 =
	sqrt (trfvec[0] * trfvec[0] + trfvec[1] * trfvec[1] +
	      trfvec[2] * trfvec[2]);
      ddist12 = (*dist12 - olddist12) / vlight / 1.e3;

/* Change the interpolation time by the difference in ranges. */
      olddt += ddist12;
      if (debug) 
        printf ("dist12 %f, ddist12 %f, old dt: %f", *dist12, ddist12, olddt);
    }

/* Inbound */
/*   TEST Sync Transponder Delay */
  seci = secutc - olddt;
  if (ephihdr.xpond_xmit_delay != 0)
    seci += ephihdr.xpond_xmit_delay / 1.e6;

  cinterp_pvac (jdint, seci, outvec, backvec,
		aberdum, aberback, &corrdum, corr23, 1, &ierr);
/*printf("bodtel %f %f %f %f %f %f art %d\n",bodtel[0],bodtel[1],bodtel[2],bodtel[3],bodtel[4],bodtel[5],arot_type);*/
/*printf("bodtel %f %f %f %f %f %f\n",bodtel[0],bodtel[1],bodtel[2],bodtel[3],bodtel[4],bodtel[5]);*/

  if (ierr != 0)
    exit(-1);

  matadd (backvec, bodtel, rfrvec, 6);

/* Apply offset from rotation to inbound vector */
  if (arot_type == 1)
    {
      matsub (rfrvec, ephftrr[0], rfrvec, 6);
    }

/* Compute range */
  *dist23 = sqrt (rfrvec[0] * rfrvec[0] + rfrvec[1] * rfrvec[1] +
		  rfrvec[2] * rfrvec[2]);

  if (debug)
    printf ("et2 %f  %f %f %f  %f %f %f  %f %f %f  %f  %f  %f\n",
	    seci, outvec[0], outvec[1], outvec[2],
	    backvec[0], backvec[1], backvec[2],
	    rfrvec[0], rfrvec[1], rfrvec[2], rnger, *dist23, dt);

}

/*-----------------------------------------------------------------------
C
C  JDTGR
C  Calculate the gregorian date from julian date.
C
C  Input
C	jdint - D.P. Whole Julian days
C	jdf   - D.P. Fractional Julian day
C 
C  Output
C	year   - Integer year
C	month  - Integer month
C	day    - Integer day
C	hour   - Integer hour
C	minute - Integer minute
C	second - D.P. second
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
jdtgr (double jdint, double jdf,
       int *year, int *month, int *day, int *hour, int *minute,
       double *second)
{
  double tday, jda1900, jdfc;

  jdfc = jdf + (jdint - (int) jdint) + 0.5e0;
  jda1900 = (int) jdint - 2415079.0e0;
  while (jdfc >= 1.e0)
    {
      jdfc -= 1.e0;
      jda1900 += 1.e0;
    }

  *hour = (int) (jdfc * 24.e0);
  *minute = (int) ((jdfc - *hour / 24.e0) * 1440.e0);
  *second = (jdfc - *hour / 24.e0 - *minute / 1440.e0) * 86400.e0;
  *year = (int) (((4.e0 * jda1900) - 1) / 1461.e0);

  tday = (int) (((4.e0 * jda1900) + 3.e0 - (*year * 1461.e0)) / 4.e0);
  *month = (int) (((5.e0 * tday) - 3.e0) / 153.e0);
  *day = (int) (((5.e0 * tday) + 2.e0 - (153.e0 * (*month))) / 5.e0);

  if (*month >= 10)
    {
      (*month) -= 9;
      (*year)++;
    }
  else
    {
      (*month) += 3;
    }
}

/*-----------------------------------------------------------------------
C  Compute rotation matrices for geodetic colatitude and longitude
C
C  - Compute station geodetic colatitude in radians
C  - For the augmented rotation matrix, set the rate of the colatitude
C    to 0.0
C  - Construct augmented rotation matrix ( [g] for geodetic colatitude,
C    positive rotation through the colatitude about the 2 axis,
C    rotation flag to +2)
C  - For the augmented rotation matrix, set the rate of the longitude
C    to 0.0
C  - Construct augmented rotation matrix ( [l] for longitude, positive
C    rotation through the longitude about the 3 axis, rotation flag
C    to +3)
C
C  Input
C	glong  - Telescope longitude
C	gedlat - Telescope geodetic latitude
C  Ouput
C	latm   - colatitude rotation matrix
C	longm  - longitude rotation matrix
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
latlonm (double longi, double gedlat, double latm[3][6], double longm[3][6])
{
  double colat[2], longit[2];

  colat[0] = twopi / 4.0 - gedlat * cdr;
  colat[1] = 0.0;
  rotmat (+2, colat, latm[0]);
  longit[0] = longi * cdr;
  longit[1] = 0.0;
  rotmat (+3, longit, longm[0]);
}

/*-----------------------------------------------------------------------
C
C
C  Compute the lunar libration rotation matrix rm using the information
C  contained in the libration (euler) angles from the jpl ephemeris.
C
C  Note: the derivative positions are not populated.
C
C  Input
C	libr - d.p. euler libration angles (radians??)
C
C  Output
C	rm   - d.p. rotation matrix
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
librate (double libr[3], double rm[3][6])
{
/*      DOUBLE PRECISION libr(3), rm(6,3)*/
  int i, j;
  double c1, c2, c3, s1, s2, s3;

  for (i = 0; i < 6; i++)
    {
      for (j = 0; j < 3; j++)
	{
	  rm[j][i] = 0.0e0;
	}
    }
  c1 = cos (libr[0]);
  c2 = cos (libr[1]);
  c3 = cos (libr[2]);
  s1 = sin (libr[0]);
  s2 = sin (libr[1]);
  s3 = sin (libr[2]);
  rm[0][0] = c3 * c1 - c2 * s1 * s3;
  rm[0][1] = c3 * s1 + c2 * c1 * s3;
  rm[0][2] = s3 * s2;
  rm[1][0] = -s3 * c1 - c2 * s1 * c3;
  rm[1][1] = -s3 * s1 + c2 * c1 * c3;
  rm[1][2] = c3 * s2;
  rm[2][0] = s2 * s1;
  rm[2][1] = -s2 * c1;
  rm[2][2] = c2;

  if (debug)
    printf ("rot: %20.13f %20.13f %20.13f\nLIBR: (jpl)\n\
 %20.13f %20.13f %20.13f\n\
 %20.13f %20.13f %20.13f\n\
 %20.13f %20.13f %20.13f\n", 
            libr[0], libr[1], libr[2], 
            rm[0][0], rm[0][1], rm[0][2], 
            rm[1][0], rm[1][1], rm[1][2], 
            rm[2][0], rm[2][1], rm[2][2]);
}

/*C-----------------------------------------------------------------------
C
C  Compute two way light travel time and apply necessary corrections
C
C  - Compute round trip range distance by adding the out bounce leg
C    distance to the bounce return leg distance.
C  - Convert round trip range distance in km to light travel time in seco
C  - Add the out bounce and bounce return corrections for geodetic 
C    curvature and convert to seconds. 
C  - Add the summed geodesic curvature corrections to the round trip
C    light travel time.
C  - Add the approximate e-m geodesic curvature correction to the round
C    trip light travel time.
C  - Multiply the corrected light travel time by the relativistic
C    clock correction to convert from coordinate to proper time.
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
littim (double dist12, double dist23, double vlight, double corr12,
	double corr23, double dtadt, double *ttltim)
{
  double range;

  range = dist12 + dist23;
  *ttltim = range / vlight + (corr12 - corr23) * 1.e-9 + 0.27e-9;
/*                                             .27=APPROX E-M REL COR */

  (*ttltim) *= dtadt;
}

/*-----------------------------------------------------------------------
C
C  Add vectors a and b with results in vector c.
C
C  Input: 
C     a = 6 element DP vector
C     b = 6 element DP vector
C     nrow = number of rows of matrices a, b and c
C
C  Output:
C     b = 6 element DP vector
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
matadd (double a[6], double b[6], double c[6], int nrow)
{
  int i;

  for (i = 0; i < nrow; i++)
    {
      c[i] = a[i] + b[i];
    }
}

/*-----------------------------------------------------------------------
C
C  Premultiply matrix b by matrix a with results in matrix c
C
C  Input:
C     a = input matrix
C     b = input matrix
C     nrow = number of rows of matrix A
C     ncol = number of columns of matrix b
C  Output:
C     c = output matrix
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
matmpy (double a[3][3], double b[3][3], double c[3][3], int nrow, int ncol)
{
  int i, j, n;
  double sum;

  for (i = 0; i < nrow; i++)
    {
      for (j = 0; j < ncol; j++)
	{
	  sum = 0.;
	  for (n = 0; n < nrow; n++)
	    {
	      sum += a[n][i] * b[j][n];
	    }
	  c[j][i] = sum;
	}
    }
}

/*-----------------------------------------------------------------------
C
C  Multiply matrices a and b with the result being in c.
C
C  Input: 
C    a - nrow x ncola DP matrix
C    b - nrow x ncola DP matrix
C    nrows - number of rows in matrices
C    ncola - number of columns in input matrices
C    ncolb - number of columns in output matrix
C   
C  Output:
C    c - nrow x ncolb DP matrix
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
matmul (double a[3][6], double b[3][6], double c[3][6], int nrows, int ncola,
//matmul (double a[ncola][nrows], double b[ncola][nrows], double c[ncola][nrows], int nrows, int ncola,
	int ncolb)
{
  int i, j, k, m1, m2, m3, m4, nrw2, imax, jmax, kmax;
  double p[3][3], v[3][3];

  nrw2 = nrows / 2;
  imax = nrw2;
  jmax = ncolb;
  kmax = ncola;

  for (j = 0; j < jmax; j++)
    {
      for (i = 0; i < imax; i++)
	{
	  p[j][i] = 0.;
	  v[j][i] = 0.;
	  for (k = 0; k < kmax; k++)
	    {
	      m1 = i;
	      m2 = k;
	      m3 = k;
	      m4 = j;
	      p[j][i] += a[m2][m1] * b[m4][m3];
	      v[j][i] +=
		a[m2][m1] * b[m4][m3 + 3] + a[m2][m1 + 3] * b[m4][m3];
	    }
	}
    }
  for (j = 0; j < jmax; j++)
    {
      for (i = 0; i < imax; i++)
	{
	  c[j][i] = p[j][i];
	  c[j][i + 3] = v[j][i];
	}
    }
}

/*-----------------------------------------------------------------------
C
C  Subtract vector b from vector a with result in vector c.
C
C  Input: 
C     a = 6 element DP vector
C     b = 6 element DP vector
C     nrow = number of rows of matrices a, b and c
C
C  Output:
C     b = 6 element DP vector
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
matsub (double a[6], double b[6], double c[6], int nrow)
{
  int i;

  for (i = 0; i < nrow; i++)
    {
      c[i] = a[i] - b[i];
    }
}

/*C-----------------------------------------------------------------------
C
C  Compute transpose of an augmented matrix
C
C  Input:
C    a - matrix to transpose
C    nrow, ncol - rows and columns in matrix a
C
C  Output:
C    a - result of transposition
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
mattrns (double a[3][6], int nrow, int ncol)
{
  int i, j, nrw2, nrw2p1;
  double b[3][3];

/* first positions */
  nrw2 = nrow / 2;
  nrw2p1 = nrw2 + 1;
  for (i = 0; i < nrw2; i++)
    {
      for (j = 0; j < ncol; j++)
	{
	  b[i][j] = a[j][i];
	}
    }
  for (i = 0; i < nrw2; i++)
    {
      for (j = 0; j < ncol; j++)
	{
	  a[j][i] = b[j][i];
	}
    }

/* then velocities */
  for (i = 0; i < nrw2; i++)
    {
      for (j = 0; j < ncol; j++)
	{
	  b[i][j] = a[j][i + nrw2];
	}
    }
  for (i = 0; i < nrw2; i++)
    {
      for (j = 0; j < ncol; j++)
	{
	  a[j][i + nrw2] = b[j][i];
	}
    }
}

/*C-----------------------------------------------------------------------
C
C  Compute pointing angles
C
C  Input
C    jd   - Julian date
C    tlrfvec - Telescope to reflector (or center of moon) vector
C    aberat  - Aberration corrections in geocentric x, y, z (d.p. m)
C    telpos  - Coordinates of telescope (d.p. radius, geoc. lat, long - m, deg)
C    gedlat  - Geodetic latitude of the telescope (d.p. degrees)
C    longi    - Longitude of telescope (d.p. degrees)
C    rotang  - Rotation angles, def'n depending on rot_type. (d.p. degrees)
C    bodftr  - coordinates of the feature on other body. E.g. lunar reflector
C		(d.p. radius, lat, long - m, degrees)
C  Output
C    lhasph  - telescope local hour angle and declination (d.p. m and degrees)
C    azalsph - telescope az and el (d.p. m and degrees)
C    ra	     - telescope right ascension (d.p. hours)
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
point (double jd, double tlrfvec[6], double aberat[3], double telpos[3],
       double gedlat, double longi, double rotang[3],
       double lhasph[6], double azalsph[6], double *ra)
{
  double bodtel[3][6], lhatel[3][6];
  double azaltel[3][6];
  double longm[3][6], latm[3][6];
  int i;

/* Apply "Stellar" aberration */
  for (i = 0; i < 3; i++)
    {
      if (legdir == 1)
	{
	  bodtel[0][i] = tlrfvec[i] + aberat[i] / 1.e3;
	}
      else
	{
	  bodtel[0][i] = tlrfvec[i] - aberat[i] / 1.e3;
	}
    }
  for (i = 3; i < 6; i++)
    {
     bodtel[0][i] = 0.e0;
     lhatel[0][i] = 0.e0;
    }

  if (debug)
    printf ("abcorr %f %f %f tlrfvec %f %f %f %f %f %f coords %f %f\n",
	    aberat[0], aberat[1], aberat[2],
	    tlrfvec[0], tlrfvec[1], tlrfvec[2], tlrfvec[3], tlrfvec[4],
	    tlrfvec[5], telpos[1], gedlat);

/* True body Fixed to topocentric ha/dec */
  latlonm (telpos[1], gedlat, latm, longm);
  if (debug)
    printf ("Telpos: %f %f %f gedlat: %f\n",
	    telpos[0], telpos[1], telpos[2], gedlat);
  mattrns (longm, 6, 3);
  matmul (longm, bodtel, lhatel, 6, 3, 1);
  if (debug)
    printf ("bodtel: %f %f %f %f %f %f lhatel: %f %f %f %f %f %f\n",
	    bodtel[0][0], bodtel[0][1], bodtel[0][2], bodtel[0][3], bodtel[0][4], bodtel[0][5],
	    lhatel[0][0], lhatel[0][1], lhatel[0][2], lhatel[0][3], lhatel[0][4], lhatel[0][5]);
  sphere (lhatel[0], lhasph);

/* topocentric ha/dec to topocentric az/el */
  mattrns (latm, 6, 3);
  matmul (latm, lhatel, azaltel, 6, 3, 1);
  sphere (azaltel[0], azalsph);

/* Clean up angles */
  azalsph[1] = 180.e0 - azalsph[1] / cdr;
  if (azalsph[1] < 0.e0)
    azalsph[1] += 360.e0;
  azalsph[2] /= cdr;

/*  The program displays RA, but it isn't important or worth the
 *  effort to get it right. Maybe it should be dropped. rlr. 05/2005 */
  *ra = (dlsid (jd, longi) + lhasph[1]) / cdr / 15.e0;
  if (*ra > 24.e0)
    (*ra) -= 24.e0;
  lhasph[1] = -lhasph[1] / cdr / 15.e0;
  lhasph[2] /= cdr;

  if (debug)
    printf ("jd, %f jdf %f, long %f, lst %f, lha %f, ra %f az %f el %f\n",
	    jd, jd - (int) jd, longi, dlsid (jd, longi) / cdr / 15.e0,
	    lhasph[1], *ra, azalsph[1], azalsph[2]);
}

/*-----------------------------------------------------------------------
C Compute augmented rotation matrix for precession
C
C
C  Author: R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
precess (double t1, double t2, double precm[3][6])
{
  double mtrx1[3][6], mtrx2[3][6], mtrx3[3][6], temp1[3][6];
  double zeta0[2], theta[2], z[2];
  double jd2000 = 2451545.e0;
  double tcent = 36525.e0;
  double t, t0;
  double a1, a2, a3, b1, b2, b3, c1, c2, c3;

  t0 = (t1 - jd2000) / tcent;
  t = (t2 - t1) / tcent;
  a1 = (-0.000139e0 * t0 + 1.39656e0) * t0 + 2306.2181e0;
  a2 = 0.30188e0 - 0.000345e0 * t0;
  a3 = 0.017998e0;
  b1 = a1;
  b2 = 1.09468e0 + 0.000066e0 * t0;
  b3 = 0.018203e0;
  c1 = (-0.000217e0 * t0 - 0.85330e0) * t0 + 2004.3109e0;
  c2 = -0.42665e0 - 0.000217e0 * t0;
  c3 = -0.041833e0;
  zeta0[0] = (((a3 * t + a2) * t + a1) * t) * casr;
  z[0] = (((b3 * t + b2) * t + b1) * t) * casr;
  theta[0] = (((c3 * t + c2) * t + c1) * t) * casr;
  zeta0[1] = ((3 * a3 * t + 2 * a2) * t + a1) / tcent * casr;
  z[1] = ((3 * b3 * t + 2 * b2) * t + b1) / tcent * casr;
  theta[1] = ((3 * c3 * t + 2 * c3) * t + c1) / tcent * casr;
  rotmat (+3, zeta0, mtrx1[0]);
  rotmat (-2, theta, mtrx2[0]);
  rotmat (+3, z, mtrx3[0]);
  matmul (mtrx2, mtrx1, temp1, 6, 3, 3);
  matmul (mtrx3, temp1, precm, 6, 3, 3);
}

/*-----------------------------------------------------------------------
C
C  Compute rectangular coordinates from spherical coordinates
C
C  Input
C	sphere - d.p. 6 element vector (radius, long, geoc lat, and 
C		 velocities. Angles in radians)
C  Output
C	xyz    - d.p. 6 element vector (x, y, z, xdot, ydot, zdot)
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
rectng (double sphere[6], double xyz[6])
{
  double sinlon, coslon, sinlat, coslat, cc, cs, sc, ss;

  sinlon = sin (sphere[1]);
  coslon = cos (sphere[1]);
  sinlat = sin (sphere[2]);
  coslat = cos (sphere[2]);
  cc = coslat * coslon;
  cs = coslat * sinlon;
  sc = sinlat * coslon;
  ss = sinlat * sinlon;
  xyz[0] = sphere[0] * cc;
  xyz[1] = sphere[0] * cs;
  xyz[2] = sphere[0] * sinlat;
  xyz[3] = sphere[3] * cc - sphere[0] * (sphere[4] * cs + sphere[5] * sc);
  xyz[4] = sphere[3] * cs + sphere[0] * (sphere[4] * cc - sphere[5] * ss);
  xyz[5] = sphere[3] * sinlat + sphere[0] * sphere[5] * coslat;
}

/*-----------------------------------------------------------------------
C
C  Construct augmented rotation matrix
C  Input:
C     kaxis   - Rotation axis: -1, -2, -3, or 1, 2, or 3
C	        Negative implies negative rotation (i.e., negate a)
C     a(2)    - angle and angle rate 
C
C  Output:
C     r (6,3) - rotation matrix
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
/* Subroutine */ int
rotmat (int kaxis, double *a, double *r__)
{
  /* Builtin functions */
  double sin (double), cos (double);

  /* Local variables */
  static int i__, j, k;
  static double a1;
  static int kp1, kp2;
  static double cosa, adot;
  static int ione;
  static double sina, adcosa, adsina;

  /* Parameter adjustments */
  r__ -= 7;
//  --a;

  /* Function Body */
  for (j = 1; j <= 6; ++j)
    {
      for (i__ = 1; i__ <= 3; ++i__)
	{
/* L1: */
	  r__[j + i__ * 6] = 0.;
	}
    }
  k = abs (kaxis);
  ione = k / kaxis;
//  a1 = ione * a[1];
//  adot = ione * a[2];
  a1 = ione * a[0];
  adot = ione * a[1];
  sina = sin (a1);
  cosa = cos (a1);
  adsina = adot * sina;
  adcosa = adot * cosa;
  kp1 = k % 3 + 1;
  kp2 = kp1 % 3 + 1;
  r__[k + k * 6] = 1.;
  r__[kp1 + kp1 * 6] = cosa;
  r__[kp1 + kp2 * 6] = -sina;
  r__[kp2 + kp1 * 6] = sina;
  r__[kp2 + kp2 * 6] = cosa;
  r__[kp1 + 3 + kp1 * 6] = -adsina;
  r__[kp1 + 3 + kp2 * 6] = -adcosa;
  r__[kp2 + 3 + kp1 * 6] = adcosa;
  r__[kp2 + 3 + kp2 * 6] = -adsina;

/* 	   the */
  return 0;
}				/* rotmat_ */

/*-----------------------------------------------------------------------
C  Compute spherical coordinates from rectangular coordinates
C
C	rho*rho = x*x + y*y
C	rho = sqrt(rho*rho)
C	r = sqrt(rho*rho + z*z)
C	long = atan(y/x)
C	lat = atan(z/rho)
C
C  Input
C	xyz - rectangular coordinates (x,y,z,dx,dy,dz)
C  Output
C	spher - Spherical coordinates (radius, lat, long, 
C		dradius, dlat, dlong)
C	        Velocities are set to 0.
C
C  Author: R. Abbot / UT Astronomy, R. L. Ricklefs / UT CSR
C  Date:  
C  History:
C-----------------------------------------------------------------------*/
void
sphere (double xyz[6], double spher[6])
{
  int i;
  double rhosq, rho;

  rhosq = xyz[0] * xyz[0] + xyz[1] * xyz[1];
  rho = sqrt (rhosq);
  spher[0] = sqrt (rhosq + xyz[2] * xyz[2]);
  spher[1] = atan2 (xyz[1], xyz[0]);
  spher[2] = atan2 (xyz[2], rho);
  for (i = 3; i < 6; i++)
    {
      spher[i] = 0.;
    }
}
