 	Program CPF_SPLIT
!********************************************************************
!	This routine selects vectors between -10 < EL > -10 for
!	the pass arc.
!
!		PAUL J. SEERY
!			23 September 1993 	Poly Format
!
!  History
!  08/25/95 - Added sun-satellite separation computation (SATIL); 
!		Ouput SUNSEP is written to .aer file.
!		Added DOYEAR to main routine so IYEAR could be passed.
!		rlr
!   04/xx/04 - Converted to use ILRS Consolidated Prediction Format as input.
!   07/13/06 - Fixed problem in which 2 satellite passes would not be
!		split into separate files if satellite did not descend to -45
!		degrees el between passes. rlr. v1.01
!   08/15/06 - Several problems solved:
!		1) wrong date given to cksun to determine whether satellite
!		   is illuminated by the sun.
!		2) wrong date given to raogu to compute Theta1. (Thanks to 
!		   Ding Jian for finding this.)
!		3) wrong date given to SATIL to compute separation of
!		   satellite from sun.
!		4) last pass from ephemeris was dropped.
!		5) inertial vectors not handled properly. (Still needs 
!		   more testing.)
!		rlr. still v1.01
!		R. Ricklefs
!********************************************************************

CC	Implicit none
	implicit double precision (a-h,o-z)

      INCLUDE  '../include/ephi.inc'
      INCLUDE  '../include/ephihdr.inc'

        COMMON /EARTH/ GM,AE,FLAT,WE
        COMMON /DOYEAR/ IDOY, IYEAR
        COMMON /MATH/ PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS
	REAL*8 PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS
	character*4 sname

	Integer*4 M, Irotate, ii, jj, j
	INTEGER*4 IXP, IYP, SUNL
	INTEGER*4 ISAND, IDOY, IYEAR, SUNCL
        INTEGER*4 MTH(12)  
	INTEGER*4 month,day,ihr,imin,mindur
	integer*4 stiyear, stidoy, stihr, stimin
        integer estart
        integer idir
        integer ircode
        integer neph
        integer ierr
        integer ift
	logical first_pass, risen
        logical starteph

	Real*8 IRV(6), XP ,YP ,THETA, SEC
	REAL*8 MJDS ,IVS(6),LAT,LONG ,HEIGHT  ,THETA1
	REAL*8 GEODET(3) ,GEOGRAF(5),SDCOS(12)
	REAL*8 AZ,EL,RANGE,xpm,ypm,FOD
	REAL*8 GM, WE, AE, FLAT
	REAL*8 SUNSEP
	REAL*8 elmax, tstart, tend
        REAL*8 outvec(6),backvec(6)
        REAL*8 aberout(3),aberback(3),corr1,corr2
        REAL*8 ephellim, tsteph(100), teneph(100)
        REAL*8 lastel

        character*59    NTAB(100)
        character*60    delstr
	character*150	mvstr
CC        character*23 dir
        character*8 dir
	
	DATA ISAND/0/
CC        DATA dir /"/controller/data/sattrk"/
        DATA dir /"data/tab"/
C
C  ** MATH COMMON **
C PI/2
        DATA PIO2  /1.570796326794897D0/
C PI
        DATA PI    /3.141592653589793D0/
C 2*PI
        DATA TWOPI /6.283185307179586D0/
C RADS/DEGREE
        DATA RDPDG /1.745329251994330D-2/
C RADS/HOUR
        DATA RDPHR /2.617993877991494D-1/
C RADS/ARC SEC
        DATA RDPAS /4.848136811      D-6/
C RADS/TIME SEC
        DATA RDPTS /7.272205217      D-5/
C
        DATA MTH/0,31,59,90,120,151,181,212,243,273,304,334/

	IXP= 0
	IYP= 0
	XP= 0
	YP= 0
        ephellim= -45.0
        starteph= .false.
        neph= 0
	call getnames()
	call EPHINITU(2,ierr)
	
        call JDTGR(jdi(1),jdf(1),iyear,month,day,
     x          ihr,imin,sec)
        IDOY = MTH(MONTH) + DAY
        if(MONTH .GT. 2) then
           j = IYEAR/4
           if((IYEAR - j*4) .eq. 0) then
              IDOY = IDOY + 1
            endif
        endif
	mindur= 0
	risen= .false.
	elmax= ephellim-5
	tstart= -1
	tend= -1
	first_pass= .true.
        if (SIC .LT. 110) ephellim= -15.0
CC	WRITE(*,121) SIC,eph_seq,IDOY,IYEAR+1900,idoy,month,day,
CC     %	ihr,imin,sec
  121	FORMAT(1X,I4,1X,I3,1X,i3,1x,i4,1x,i3,1x,4(i2,1x),f6.3)
	call COMBLX

	READ(1,100) sname,LAT,LONG,HEIGHT
 100	FORMAT(1x,/,/,/,/,22X,A4,14X,3F12.5)
CC	write(*, 169) sname, LAT, LONG, HEIGHT
 169	format(5x, A4, 3f12.5)
	GEODET(1) = LAT
	GEODET(2) = LONG
	GEODET(3) = HEIGHT
	CALL STATION(GEODET, GEOGRAF ,AE ,FLAT, SDCOS)
	GEOGRAF(4) = LAT
	GEOGRAF(5) = LONG
CC	write(*,*) sname,LAT,LONG,HEIGHT

  200	CONTINUE
CC	write(*,*) "nv=",nv

CC	DO jj = 1,nv
CC          write (*,*) jj,jdi(1),jdf(1),jds(jj),
CC     &		outv(jj,1),outv(jj,2),outv(jj,3)
CC        ENDDO

C First, create the schedule of rise, set
        ift= int((jdi(nv)- jdi(1) + (jdf(nv)- jdf(1)))*1440.d0)
CC        DO jj = 1,nv*ephsep/60
        DO jj = 1,ift

        call JDTGR(jdi(1),jdf(1)+ jj/1440.d0,iyear,month,day,
     x          ihr,imin,sec)
        MJDS= jdi(1)- 2400000.5d0+jdf(1)+ jj/1440.d0
        IDOY = MTH(MONTH) + DAY
        if(MONTH .GT. 2) then
           j = IYEAR/4
           if((IYEAR - j*4) .eq. 0) then
              IDOY = IDOY + 1
            endif
        endif
        FOD = MOD(MJDS,1.0d0)
CC	call raogu (jdi(jj)+jdf(jj), theta1)
CC	write(*,*) "theta, reffr", theta1, ref_frame
        call interp_pvac (jdi(1), jdf(1)*86400.d0+jj*60.d0, 
     &     outvec,backvec,aberout,aberback,corr1,corr2,1,ircode)
        if (ircode .NE. 0) go to 370
        irv(1)= outvec(1)
        irv(2)= outvec(2)
        irv(3)= outvec(3)
        irv(4)= 0
        irv(5)= 0
        irv(6)= 0
CC	write(*,*) jdi(1), jdf(1)*86400.d0+jj*60.d0,"irv=",irv
C
C   CHECK SUN LIT AREA
CC	CALL CKSUN(IRV, IYEAR+1900, IDOY, FOD, SUNL)
CC	write(*,*) "sunl",sunl

C   Convert body fixed to inertial frame for sun check.
	call raogu (JDI(1)+JDF(1)+JJ/1440.D0, theta1)
        if (ref_frame .eq. 0) then 
	  Irotate = 0
	  CALL IVIRV_PC(irv, IVS,THETA1,XP,YP ,Irotate)
        else
          do j=1,6
            ivs(j)= irv(j)
          enddo
	  Irotate = 1
	  CALL IVIRV_PC(ivs, irv,THETA1,XP,YP ,Irotate)
        endif
	CALL CKSUN(ivs, IYEAR+1900, IDOY, FOD, SUNL)
C
C	CONVERT INERTIAL TO BODY FIXED
C	For the moon the az.el, and especially range are very low
C         precision, but is OK for scheduling.
        lastel= el
	CALL look (irv, GEOGRAF, AZ, EL, RANGE)
CC	write(*,*) "az,el,r",az,el,range
C
C   CHECK SUN LIT AREA
	CALL SATIL(MJDS+2400000.5d0,0.d0,
     x		long*RDPDG,lat*RDPDG,
     x		az*RDPDG,el*RDPDG,SUNSEP)
	SUNCL= 0
	IF (SUNSEP .lt. 35.) SUNCL= 1
CC	write(*,*) "sunsep, suncl",sunsep,suncl

	IF(ISAND.EQ.1) GO TO 300
CC        IF(EL .GT. ephellim) GO TO 370
        IF(EL .GT. ephellim .and. 
     x		.not.(tend .gt. -1 .and. el .gt. lastel))
     x		 GO TO 370
	ISAND =1
300	CONTINUE
CC	write(*,*) "el= ",el
CC	write(*,*) "el, ephellim, tend, lastel",el,ephellim,tend,lastel
        IF(EL .GT. ephellim .and. 
     x		.not.(tend .gt. -1 .and. el .gt. lastel)) THEN
CC        IF(EL .GT. ephellim) THEN
        if (.not.starteph) then
          starteph=.true.
          neph= neph+ 1
          tsteph(neph)= mjds
CC	write(*,*) "neph=",neph,tsteph(neph)
        endif
        teneph(neph)= mjds

	if (el .gt. 10.0 .and. tstart .eq. -1) then
          tstart= mjds
	  stiyear= iyear
	  stidoy= idoy
	  stihr= ihr
	  stimin= imin
c	write(*,*) "jydhm=",mjds,stiyear,stidoy,stihr,stimin
        endif
CC	write(*,*) "tstart,el,tend",tstart,el,tend
	if (tstart .gt. -1 .and. el .lt. 10.0 .and. tend .eq. -1) 
     x		tend= mjds

	if (.not.risen) then
	  risen= .true.
	  if (first_pass) WRITE(3,120) SIC,eph_seq,IDOY,IYEAR+1900

	  first_pass= .false.
  120	  FORMAT(1X,I4,1X,I4,1X,i3,1x,i4)
	endif
C
C	OUTPUT THE IRV AND THE RANGING DATA TO UNIT 3
C
	  if (el .gt. elmax) elmax= el
CC	  WRITE(*,1000)MJDS,(ivs(M),M=1,6)
	  WRITE(3,1000)MJDS,(ivs(M),M=1,6)
 1000	  FORMAT(1X,F17.10,3(1X,F18.6),3(1X,F18.9))
	  WRITE(3,1100) AZ,EL,RANGE*2/0.2997925e9,xpm,ypm,SUNL, SUNCL
CX	  WRITE(3,1100) AZ,EL,RANGE,xpm,ypm,SUNL, SUNCL
CC	  WRITE(*,1100) AZ,EL,RANGE,xpm,ypm,SUNL, SUNCL
1100	  FORMAT(1X,F8.4,1X,F8.4,1X,F18.10,2(1x,f8.0),1x,i1,1x,i1)
CX1100	  FORMAT(1X,F8.4,1X,F8.4,1X,F18.3,2(1x,f8.0),1x,i1,1x,i1)

	else if (risen) then
CC	write(*,*) "risen:",dir,year,stidoy,stihr,stimin
          starteph=.false.
	  risen= .false.
	  mindur= (tend- tstart)*1440.0 + 0.5
          write (NTAB(neph),1111) DIR,IYEAR+1900, stIDOY, stihr, 
     x           stIMIN, SIC, MINDUR, eph_source
 1111     FORMAT(a,'/tabl_y',I4,'d',I3.3,'t',
     x          2I2.2,'_',I4.4,'.',I4.4,'.',a3)
CC	write(*,*) "File:",neph,NTAB(neph)
	  tstart= -1
	  tend= -1
CC	write(*,*) "elmax= ",elmax
	  if (elmax .lt. 20) then
            neph= neph- 1
	  else
	    mindur= 0
	  endif
          elmax= ephellim-5
	ENDIF
370	CONTINUE
	ENDDO

C   Is this always OK??
C  Pass not complete by end of ephemeris
	if (risen) neph= neph- 1

CC	do jj=1,neph
CC	write(*,*) "neph", jj,tsteph(jj),teneph(jj),NTAB(jj)
CC        enddo

        ii= 1
        estart= 0
        DO jj = 1,nv
        call JDTGR(jdi(jj),jdf(jj),iyear,month,day,
     x          ihr,imin,sec)
        IDOY = MTH(MONTH) + DAY
        if(MONTH .GT. 2) then
                j = IYEAR/4
                if((IYEAR - j*4) .eq. 0) then
                        IDOY = IDOY + 1
                endif
        endif
        MJDS= jdi(jj)- 2400000.5d0+jdf(jj)
c	write(*,*) "mjds= ",mjds

        if (mjds .ge. tsteph(ii) .and. mjds .lt. teneph(ii)) then
          if (estart .eq. 0) then
            OPEN(UNIT=4, STATUS='UNKNOWN', FILE="tabl_temp", ERR=9997)
CC	write(*,*) "begineph mjds", MJDS
C           Be sure round-off doesnt give us 2 headers...
            call begineph(2,4)
            estart= 1
          endif

CC	write(*,*) "nexteph mjds", mjds
          call nexteph(2,4,MJDS)
        endif
        if (mjds .gt. teneph(ii)) then
          OPEN(UNIT=5, STATUS='UNKNOWN', FILE=NTAB(ii), ERR=9998)
CC	write(*,*) "endeph mjds, teneph, ii", mjds,teneph(ii),ii
          call endeph(4,5)
          estart= 0
	  CLOSE (UNIT=4)
	  CLOSE (UNIT=5)
          ii= ii+ 1
CC	write(*,*) "tst, ten=",tsteph(ii),teneph(ii)
        endif
        if (ii .gt. neph) go to 9999
        
9995	ENDDO
	goto 9999
9997	CONTINUE
	WRITE(*,*) "Could not open file: tabl_temp"
	goto 9999
9998	CONTINUE
	WRITE(*,*) "Could not open file:",NTAB(ii)
9999	CONTINUE
	END

*************************************************************************
*									*
*	SUBROUTINE STATION(GEODET, GEOGRAF)
*									*
*PURPOSE								*
*	THIS SUBROUITNE WILL CONVERT STATION GEODITICS			*
*	TO EARTH FIXED COORDINATES (X,Y,Z)				*
*									*
*DESCRIPTION OF PARAMETERS						*
*  ARRAYS:								*
*	GEODET= STATION COORDINATES(INPUT)                              *
*	GEOGRAF=3 XYZ STATION COORDINATES (OUTPUT)			*
*  VARIABLES:								*
*	LAMBDA=EAST LONGITUDE MEASURED EASTWARD FROM			*
*	       PRIME MERIDIAN IN DEGREES				*
*	HEIGHT=ELEVATION IN METERS					*
*	PHI   = GEODETIC LATITUDE IN DEGREES				*
*	AE= EQUATORIAL RADIUS IN METERS					*
*	FLAT = INVERSE FLATTENING CONSTANT				*
*									*
*									*
*COMMENTS								*
*	CJN 8/83							*
*	UPDATE REJ 10/84                                                *
*	UPDATE BRION CONKLIN BFEC/DSG/OAS 3/90                          *
*************************************************************************
	SUBROUTINE STATION(GEODET, GEOGRAF ,AE ,INVERSE_FLAT, SDCOS)

	IMPLICIT NONE

	REAL*8 HEIGHT, AE, G1, G2, FLAT, INVERSE_FLAT, PHI,
     1	       LAMBDA, DENOM
	REAL*8 GEODET(3),GEOGRAF(3),SDCOS(12)

        COMMON /MATH/ PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS
	REAL*8 PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS

	PHI=GEODET(1)*RDPDG
	LAMBDA=GEODET(2)*RDPDG
	HEIGHT=GEODET(3)
*
	FLAT = 1.0D00/INVERSE_FLAT
*
*FIND G1 AND G2
*
	DENOM=1.0D0-((2*FLAT-(FLAT*FLAT))*(DSIN(PHI)**2))
	G1=(AE/DSQRT(DENOM)) + HEIGHT
CC	write(*,*) "flat",flat,"ae",ae,"g1",g1,"height",height
	G2=((AE*((1.0D0-FLAT)**2))/DSQRT(DENOM)) + HEIGHT
*
*FIND X,Y,Z,TRANSFORMATION FROM GEODETIC TO EARTH FIXED
*
	GEOGRAF(1)=G1*DCOS(PHI)*DCOS(LAMBDA)
	GEOGRAF(2)=G1*DCOS(PHI)*DSIN(LAMBDA)
	GEOGRAF(3)=G2*DSIN(PHI)
*
	RETURN
	END



*************************************************************************
*									*
*PURPOSE								*
*	THIS SUBROUTINE WILL TRANSFORM inertial and rotating vectors    *
*									*
*DESCRIPTION OF PARAMETERS						*
*       INPUT:                                                          *
*               V1 = 6 COMPONENT VECTOR TO BE CONVERTED                 *
*		THETA=TRUE OF DATE RIGHT ASCENSION OF GREENWICH		*
*		Irotate = 1, then the transformation will               *
*				be from inertial to the rotating frame. *
*			= 0, then the opposite takes place.             *
*		XP,YP = X OF POLE ,Y OF POLE                            *
*	OUTPUT:                                                         *
*		V2 = 6 COMPONENT VECTOR CONVERTED                       *
*	ARRAYS:								*
*		ROTMAT = 3X3 ROTATION MATRIX for position vector        *
*		ROTMATdot = 3X3 ROTATION MATRIX for velocity vector     *
*									*
*SUBROUTINES AND FUNCTIONS						*
*									*
*COMMENTS								*
*	REJ 9/86                                                        *
*************************************************************************
	SUBROUTINE IVIRV_PC(V1, V2,THETA,XP,YP ,Irotate)

	IMPLICIT NONE
        COMMON /EARTH/ GM,AE,FLAT,WE
	INTEGER*4  I, J, Ind1, Ind2, Irotate

	REAL*8 V1(6), V2(6), ROTMAT(3,3), RotmatDot(3,3), Vtemp(3)
	REAL*8 THETA ,Xp, Yp
	REAL*8 GM, WE, AE, FLAT



*
*COMPUTE ROTATION MATRIX for the position vector
*
	rotmat(1,1) = dcos(Theta)
	rotmat(1,2) = dsin(Theta)
	rotmat(1,3) = Xp
	rotmat(2,1) = -rotmat(1,2)
	rotmat(2,2) = rotmat(1,1)
	rotmat(2,3) = -Yp
	rotmat(3,1) = -Xp * rotmat(1,1) - Yp * rotmat(1,2)
	rotmat(3,2) = -Xp * rotmat(1,2) + Yp * rotmat(1,1)
	rotmat(3,3) = 1.0d00
*
*COMPUTE ROTATION MATRIX for the velocity vector
*
	ROTMATDOT(1,1) = -rotmat(1,2)
	ROTMATDOT(1,2) = rotmat(1,1)
	ROTMATDOT(1,3) = 0.d0
	ROTMATDOT(2,1) = -rotmat(1,1)
	ROTMATDOT(2,2) = -rotmat(1,2)
	ROTMATDOT(2,3) = 0.D0
	ROTMATDOT(3,1) = Xp * rotmat(1,2) - Yp * rotmat(1,1)
	ROTMATDOT(3,2) = -Xp * rotmat(1,1) - Yp * rotmat(1,2)
	ROTMATDOT(3,3) = 0.0d00
*
*MATRIX MULTIPLY
*
	DO J = 1,3
	  V2(J) = 0.0D00
	  V2(J+3) = 0.0D00
	  Vtemp(j) = 0.d0
	  DO I = 1,3
	    Ind1 = j
	    Ind2 = i
	    If (Irotate.EQ.0) then
	      Ind1 = i
	      Ind2 = j
	    End if
	    V2(J) = V1(I) * ROTMAT(Ind1, Ind2) + V2(J)
	    V2(J+3) = V1(I+3) * ROTMAT(Ind1, Ind2) + V2(J+3)
	    Vtemp(j) = V1(i) * RotmatDot(Ind1, Ind2) + Vtemp(j)
	  End do
	  V2(j+3) = V2(j+3) + WE * Vtemp(j)
	End do

	RETURN
	END

!******************************************************************************
	subroutine look (IRV_VECTOR, GEOGRAF, azimuth, elevation, range)
!******************************************************************************
!
!
!
!	PURPOSE
!
!	ARGUMENTS
!	---------
!	IRV_VECTOR   = earth centered satellite vector (input)
!
!	MJD = time tag of IV_VECTOR in MEAN JULIAN DAYS + FRACTION OF DAY
!
!	GEOGRAF = earth centered station vector (input)
!
!	LH    = local tanget satellite vector
!
!	azimuth   = azimuth between IV_VECTOR and sta_vec (output)
!
!	elevation = elevation    "         "               "
!
!	range     = range        "         "               "
!
!       IMPORTANT VARIABLES
!	-------------------
!
!	SUBROUTINES AND FUNCTIONS USED
!	______________________________
!
!	PROGRAMMER & CREATION DATE
!	--------------------------
!	RICKY JOHNSON 1/85
!
!	MODIFICATIONS
!	-------------
!******************************************************************************
	implicit none

	integer*4 I,J
	real*8 IRV_VECTOR(6), GEOGRAF(5), L(3), PHI,
     *         LH(3), azimuth, elevation, range, sum, LAMBDA,
     *         RHO(3), M(3,3)

        COMMON /MATH/ PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS
	REAL*8 PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS

	PHI=GEOGRAF(4)*RDPDG
	lambda = geoGRAF(5)*RDPDG
	SUM = 0.0
	do i = 1,3
	  LH(i) = 0.0d00
	  RHO(I) = IRV_VECTOR(I) - GEOGRAF(I)
	  SUM = SUM + RHO(I)**2
	end do
	RANGE = DSQRT(SUM)
	DO I=1,3
	  L(I) = RHO(I)/RANGE
	END DO

	m(1,1) =  dsin(PHI) * dcos(lambda)
	m(1,2) =  dsin(PHI) * dsin(lambda)
	m(1,3) = -dcos(PHI) 
	m(2,1) = -dsin(lambda)
	m(2,2) =  dcos(lambda)
	m(2,3) =  0.0d00
	m(3,1) =  dcos(PHI) * dcos(lambda)
	m(3,2) =  dcos(PHI) * dsin(lambda)
	m(3,3) =  dsin(PHI)

	do i = 1,3
	  do j = 1,3
		LH(i) = m(i,j) * L(J) + LH(i)
	  end do
	end do

	ELEVATION = DASIN(LH(3))/RDPDG
	SUM = DSQRT(1.0 - LH(3)**2)
	AZIMUTH = DATAN2(LH(2)/SUM,-LH(1)/SUM)/RDPDG
	IF (AZIMUTH.LT.0.0) AZIMUTH = AZIMUTH + 360.

	return
	end

C *************************************************************************
	SUBROUTINE COMBLX
        IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C       INITIALIZE COMMON
C
C                        11-JULY-91
C                                   PAUL J. SEERY
C
C       INCLUDES DYNAMIC CALCULATION FOR EARTH ROTATION RATE
C *************************************************************************
C
        COMMON /EARTH/ GM,AE,FLAT,WE
        COMMON /JHARM/ HJ2,HJ3,HJ4
        COMMON /LIGHT/ C
        COMMON /CONST/ TWOPI,PI,DTR,RTD
        COMMON /DOYEAR/ IDOY, IYEAR
C
C               GM = M**3/SEC**2
C               AE = M
        GM = 3.9860044D14                   
        AE = 6.378137D6 		! Texas model            
        FLAT = 298.255D0
C
        PI = 4.0D0*DATAN(1.0D0)
        TWOPI = 2.0D0*PI
        DTR = PI/180.0D0
        RTD = 180.0D0/PI
C
C               WE = RAD/SEC
        DOY = IDOY
        CALL JULD(DOY,IYEAR+1900,DJUL)
        T = DJUL/36525.D0
        R = 1.002737909351D0 + T*(5.9006D-11 - T*5.9D-15)
        WE = PI*R/43200.0D0
C
C               HARMONICS (J2,J3,J4)
        HJ2 = 1.0826271D-3          
        HJ3 = -2.53589D-6           
        HJ4 = -1.62462D-6           
C
C               C = M/SEC (SPEED-OF-LIGHT)
        C = 2.99792458D8            
C
        RETURN
        END

C --------------------------------------------------------------------
C
C  PROGRAM NAME
C       JULD
C
C  ENGLISH NAME
C       DELTA JULIAN DAYS PAST EPOCH
C
C  VERSION
C       .3
C
C  PURPOSE
C       TO RETURN THE NUMBER OF JULIAN DAYS PAST
C       EPOCH (JAN 1.5 2000) GIVEN YEAR AND CURRENT DAY
C
C  LANGUAGE
C       FORTRAN
C
C  COMPUTER
C       MODCOMP-II
C       PDP-11/24
C       PDP-11/23
C	MIK-11/2
C	PC (COMPAQ, GATEWAY, UNIQ)
C	HP-9000/300
C
C  METHOD
C       REFERENCE: ASTRONOMICAL ALMANAC
C
C  CALLING SEQUENCE
C       CALL JULD(DOY,IYR,DJUL)
C
C  INPUT VARIABLES
C    DOY  = DAY-OF-YEAR (INCLUDES FRACTION OF DAY)
C    IYR  =  YEAR (4 digit)
C
C  INTERMEDIATE VARIABLES
C       NONE
C
C  OUTPUT VARIABLES
C    DJUL = DELTA(JULIAN) DAYS PAST EPOCH
C
C  DEVELOPMENT HISTORY
C       NAME            DATE            DESCRIPTION
C   PAUL J. SEERY    8 AUG 83
C   PAUL J. SEERY    1 NOV 89   ALLOW FOR F.O.D. IN (DOY)
C   PAUL J. SEERY   27 APR 90   EPOCH TO: JAN 1.5 2000
C
C --------------------------------------------------------------------
      SUBROUTINE JULD (DOY,IYR,DJUL)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C     DELTA JULIAN DAYS PAST  JAN 1.5 , 2000  (245 1545.0)
C
C            INPUT      DOY       = CURRENT DAY (INCLUDES F.O.D.)
C                       IYR       = YEAR (19xx)
C
C            OUTPUT     DJUL      = DELTA (JULIAN) DAYS FROM EPOCH
C
      AIY    = IYR - 2000
      BIY    = AIY/4.0 
      ICIY   = BIY
      CIY    = ICIY 
      IF (BIY.GT.CIY) CIY = CIY + 1.0
      DJUL   = (AIY - CIY)*365. + (CIY*366.) + DOY - 1.5
      RETURN
      END

C ---------------------------------------------------------------------
C 	* SUNLIT DETERMINATION SUBROUTINES *
C ---------------------------------------------------------------------
	SUBROUTINE CKSUN(IV,IYEAR,IDOY,FOD,SUNL)
C +++++
C +++++ 	IV is the satellite coords in earth
C +++++				centered inertial (m)
C +++++		FOD is fraction of day
C +++++		IDOY   is day of year
C +++++		IYEAR  is year (4 digits)
C +++++		SUNL is sunlit (1/0 = yes/no)
C +++++
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
	DOUBLE PRECISION IV(6)
	INTEGER IYEAR,IDOY,SUNL

	DOUBLE PRECISION VISUN(3)
C +++++ the following units are in meters
	DATA AE/6378.14D3/,AS/696000.D3/
C
	CALL SUNXYZ(IYEAR,IDOY,FOD,VISUN)
C
	RSAT=DSQRT(IV(1)**2 + IV(2)**2 + IV(3)**2)
	RSUN=DSQRT(VISUN(1)**2+VISUN(2)**2+VISUN(3)**2)
	RX=AE*RSUN/(AS-AE)
C		if satellite is far enough away, then always sunlit
	IF(RSAT.GE.RX) THEN
	   SUNL = 1
	ELSE
	   XX=VISUN(1)*RX/RSUN
	   XY=VISUN(2)*RX/RSUN
	   XZ=VISUN(3)*RX/RSUN
	   TEST=-(XX*IV(1)+XY*IV(2)+XZ*IV(3))
C		if the satellite is on the same side of the earth
C		as the SUN then it's sunlit! ...
	   IF(TEST.LT.0.D0) THEN
		SUNL = 1
	   ELSE
C		otherwise must check if satellite is within shadow
C		of earth...
		SINDU=(AS-AE)/RSUN
		COSDU=DSQRT(1.D0-SINDU*SINDU)
		PX=XX + IV(1)
		PY=XY + IV(2)
		PZ=XZ + IV(3)
		RP=DSQRT(PX*PX+PY*PY+PZ*PZ)
		COSTH=(PX*XX + PY*XY + PZ*XZ)/(RP*RX)
C			if the angle to the satellite is greater than
C			the sun's shadow angle (DU) then its sunlit.
		IF(COSTH.LT.COSDU) THEN
			SUNL = 1
		ELSE
			SUNL = 0
		ENDIF
	   ENDIF
	ENDIF
	RETURN
	END

	SUBROUTINE SUNXYZ (IYEAR,IDOY,FOD,VISUN)
C
C   COMPUTE THE RECTANGULAR COORDINATES OF THE SUN FROM THE CENTER
C	OF THE EARTH IN INERTIAL COORDINATES.
C		FOD  is fraction of day
C		IDOY  is day of year
C		IYEAR is year (4 digits)
C		VISUN is sun coords in earth-centered-rotating (m)
C
	IMPLICIT DOUBLE PRECISION (A-H,O-Z)
        COMMON /CONST/ TWOPI,PI,DTR,RTD
	DIMENSION VISUN(3)
C +++++ units: (A.U.)meters, (EPS)degrees, (P)rads/second
	DATA AU/1.49598845D11/
C
	DOY = IDOY + FOD
	CALL JULD(DOY,IYEAR,DJ)
C
C		mean longitude of SUN (corrected for aberration)
	XSML = 280.460 + 0.9856474*DJ
	XSML = DMOD(XSML,360.D0)
C
C		mean anomaly
	XSMA = 357.528 + 0.9856003*DJ
	XSMA = DMOD(XSMA,360.D0)
	SINMA= DSIN(XSMA*DTR)
	SINMA2=DSIN(2.D0*XSMA*DTR)
	COSMA =DCOS(XSMA*DTR)
	COSMA2=DCOS(2.D0*XSMA*DTR)
C
C		ecliptic longitude
	XSEL = XSML + 1.915*SINMA+0.020*SINMA2
C
C		obliquity of ecliptic (ecliptic latitude=0.D0)
	XSOE = 23.439 - 0.0000004*DJ
C
C		distance of SUN from EARTH
	RSUN = AU*(1.00014 - 0.01671*COSMA - 0.00014*COSMA2)
C
C		rectangular coordinates of SUN
	VISUN(1)=RSUN*DCOS(XSEL*DTR)
	VISUN(2)=RSUN*DCOS(XSOE*DTR)*DSIN(XSEL*DTR)
	VISUN(3)=RSUN*DSIN(XSOE*DTR)*DSIN(XSEL*DTR)
C	
	RETURN
	END

	SUBROUTINE RAOGU (UTCJD, HA0)
C
C...VERSION 811015
C
C...PURPOSE: TO COMPUTE THE RIGHT ASCENSION OF THE GREENWICH MERIDIAN FROM
C	     THE MEAN EQUINOZ OF DATE.  THIS VERSION USES UTC AS THE INPU
C	     TIMESCALE AND NEGLECS THE DIFFERENCE BETWEEN UTC AND UT1.
C
C...INPUT:   IRVTIM(5) = DATE ARRAY USED IN CLACULATING JULIAN DATE.  COMPONENTS
C			 ARE, IN ORDER: (YEAR,MONTH,DAY,HOUR,MINUTE).
C	     SECS	=FLOATING POINT SECONDS COMPONENT OF ABOVE DATE ARRAY.
C...OUTPUT: (ALL IN COMMON /TIMIRV/)
C		UTCJD	= JULIAN DATE, IN COORINATE UNIVERSAL TIME.
C		ETJD	= JULIAN DATE, IN APPROXIMATE EPHEMERIS TIME.
C		HA0	= HOUR ANGLE OF CONVENTIONAL ZERO MERIDIAN
C
	IMPLICIT REAL*8(A-H,O-Z)
	IMPLICIT INTEGER*4 (I-N)
CC	COMMON /TIMIRV/ IRVTIM(5),SECS,UTCJD,ETJD,HA0
	COMMON /MATH/ PIO2,PI,TWOPI,RDPDG,RDPHR,RDPAS, RDPTS
	REAL*8 PIO2, PI, TWOPI, RDPDG, RDPHR, RDPAS, RDPTS

C
	DATA AL0,AL1,AL2/1.7399358937175E0, 1.720279266007E-2,
     1			 5.0640897110023E-15/
C   ETMTAI = (EPHEMERIS TIME - INTERNATIONAL ATOMIC TIME), IN DAYS:
	DATA ETMTAI/0.0003725/
	DATA REFEP, UT1REF,DUT1/2440224.5E0,-6.981E0,-0.00286E0/
C	type 111
111	format (1x,'Entering : RAOGU')
C
C
C...COMPUTE JULIAN DATE IN EPHEMERIS TIME:
	DTDAY = UTCJD - REFEP
C   UT1MTA = (UNIVERSAL TIME 1 - INTERNATIONAL AOMIC TIME), IN DAYS:
	UT1MTA = (UT1REF + DUT1*DTDAY)/86400.0E0
C   UT1MET = (UNIVERSAL TIME 1 - EPHEMERIS TIME):
	UT1MET = UT1MTA - ETMTAI
C   ETJD = JULIAN DATE, IN EPHEMERIS TIME (APPROXIMATE ONLY, SINCE UT1MET IS
C		USED INSTEAD OF UTCMET):
	ETJD = UTCJD - UT1MET
C
C
C...COMPUTE JULIAN DAYS ELAPSED FROM 1900 JAN 0 DAYS 12 HOURS:
	TU1901 = UTCJD - 2415020.0E0
	TU1902 = TU1901 * TU1901
C
C
C...ANGLE FROM MEAN EQUINOZ TO GREENWICH MERIDIAN AT CURRENT TIME:
	ALPHA1 = AL0 + AL1*TU1901 + AL2*TU1902
C
C...FRACION OF DAY SINCE 0 HOURS UT1:
	ARG = TU1901 - 0.5
	UT1 = MOD(ARG,1.0D0)
C
C...HOUR ANGLE OF CONVENTIONAL ZERO MERIDIAN (BETWEEN 0 AND TWOPI)
	ARG = ALPHA1 + UT1*TWOPI
	HA0 = MOD(ARG,TWOPI)
	RETURN
	END

      SUBROUTINE JDTGR(JDINT,JDF,YEAR,MONTH,DAY,HOUR,MINUTE,SECOND)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

C  CALCULATE THE GREGORIAN DATE FROM JULIAN DATE.
C
C  REVISIONS:
C
C$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

      DOUBLE PRECISION JDINT,JDF,TDAY,JDA1900,JDFC
      INTEGER YEAR,MONTH,DAY,HOUR,MINUTE
C
      JDFC = JDF + (JDINT-DINT(JDINT)) + 0.5D0
      JDA1900 = DINT(JDINT) - 2415079.0D0
2      IF (JDFC.LT.1.D0) GO TO 5
        JDFC  = JDFC - 1.D0
        JDA1900= JDA1900 + 1.D0
        GO TO 2

 5    HOUR   = IDINT(JDFC*24.D0)
      MINUTE = IDINT((JDFC - HOUR/24.D0)*1440.D0)
      SECOND = (JDFC - HOUR/24.D0 - MINUTE/1440.D0)*86400.D0
      YEAR   = IDINT(((4.D0*JDA1900)-1)/1461.D0)
 
      TDAY   = DINT(((4.D0*JDA1900)+3.D0- (YEAR*1461.D0))/4.D0)
      MONTH  = IDINT(((5.D0*TDAY)-3.D0)/153.D0)
      DAY    = IDINT(((5.D0*TDAY)+2.D0 - (153.D0*MONTH))/5.D0)
 
      IF(MONTH.LT.10) GO TO 10
         MONTH = MONTH - 9
         YEAR  = YEAR + 1
         GO TO 99
 10      MONTH = MONTH + 3
 
 99   CONTINUE
C
         END

      subroutine begineph(unitin, unitout)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      common /ehdrstr/ headers, nh
      character headers(10)*100
      integer unitin, unitout
      integer nh
      integer trimlen
      logical first
      data first /.true./
      save first

      if (first) then
        rewind (unitin)
        first= .false.
        nh= 0
   10   nh= nh+1
        READ (unitin,20,end=120) headers(nh)
   20   FORMAT (a100)
        IF (headers(nh)(1:2).ne."h9" .AND. headers(nh)(1:2).ne."H9") 
     &		go to 10
      endif

      do i=1,nh
        WRITE (unitout,21) headers(i)(1:trimlen(headers(i)))
   21   FORMAT(a)
      enddo
      RETURN
 120  write(*,*) "Too few headers"
      stop
      END

      subroutine nexteph(unitin,unitout,MJDS)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      common /ephtimes/ sttjdi, sttjdf, endjdi, endjdf
      DOUBLE PRECISION sttjdi, sttjdf, endjdi, endjdf

      character detail*200
      double precision mjd,sod, jdi, jdf, MJDS, MJDT
      integer unitin, unitout
      integer trimlen
      integer idir
      logical wstart
      logical first
      data first /.true./
      save first
      save detail
 
      wstart= .false.
      if (.not.first) go to 15
      first= .false.
   10 READ (unitin,11,end=120) detail
   11 FORMAT (a200)
   15 IF (detail(1:2).ne."10") go to 20
        READ (detail(3:100),*) idir,mjd,sod
        if (idir .ne. 1 .and.idir .ne. 0) go to 20
        if (wstart) THEN
          endjdi= jdi
          endjdf= jdf
          RETURN
        ENDIF
        jdi=mjd+2400000.0d0
        jdf=sod/86400.d0+0.5d0
        IF (jdf.ge.1.0d0) THEN
          jdi=jdi+1.0d0
          jdf=jdf-1.0d0
        ENDIF
        MJDT= jdi- 2400000.5d0+jdf
        if (MJDS .EQ. MJDT) THEN
          wstart= .true.
        ENDIF
   20 if (wstart) then
        WRITE (unitout,21) detail(1:trimlen(detail))
   21   FORMAT(a)
      endif
      go to 10
      
  120 RETURN
      END

      subroutine endeph(unitin, unitout)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

      INCLUDE  '../include/ephihdr.inc'

      common /ephtimes/ sttjdi, sttjdf, endjdi, endjdf
      DOUBLE PRECISION sttjdi, sttjdf, endjdi, endjdf

      character line*200
      double precision mjd,sod
      integer unitin, unitout
      integer trimlen, idir

C  Getting the starting date and time.
      rewind (unitin)
   10 READ (unitin,11,end=15) line
   11 FORMAT (a200)
      IF (line(1:2).ne."10") go to 10
        READ (line(3:200),*) idir,mjd,sod
        sttjdi=mjd+2400000.0d0
        sttjdf=sod/86400.d0+0.5d0
        IF (sttjdf.ge.1.0d0) THEN
          sttjdi=sttjdi+1.0d0
          sttjdf=sttjdf-1.0d0
        ENDIF
   15 rewind (unitin)

C now read the H2 record and correct the start and end times.
   20 READ (unitin,11,end=120) line
      IF (line(1:2).ne."h2" .AND. line(1:2).ne."H2") THEN
          WRITE (unitout,21) line(1:trimlen(line))
   21     FORMAT(a)
          go to 20
      ENDIF
      call JDTGR(sttjdi,sttjdf+1.0d-10,sttpyear,sttpmonth,sttpday,
     x          sttphour,sttpmin,dsttpsec)
       sttpyear= sttpyear+ 1900
       sttpsec= dsttpsec
      call JDTGR(endjdi,endjdf+1.d-10,endpyear,endpmonth,endpday,
     x          endphour,endpmin,dendpsec)
       endpyear= endpyear+ 1900
       endpsec= dendpsec
      WRITE (unitout,40) line(1:2),cospar_id,sic,norad,
     &   sttpyear,sttpmonth,sttpday,sttphour,sttpmin,sttpsec,
     &   endpyear,endpmonth,endpday,endphour,endpmin,endpsec,
     &   ephsep,compat,tar_type,ref_frame,rot_type, cofm_app
   40 FORMAT (a2,1x,i8,1x,i4,1x,i8,1x,2(i4,5(1x,i2),1x),i5,1x,i1,
     &   1x,i1,1x,i2,1x,i1,1x,i1)
      go to 20
  120 WRITE (unitout,50)
   50 FORMAT ("99")
      RETURN

      END
